@@ -13,17 +13,17 @@
 %                          Write A Histogram Image.                           %
 %                                                                             %
 %                              Software Design                                %
-%                                John Cristy                                  %
+%                                   Cristy                                    %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -40,7 +40,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/property.h"
+#include "magick/artifact.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/cache.h"
@@ -50,17 +50,23 @@
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/geometry.h"
+#include "magick/histogram.h"
 #include "magick/image-private.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
-#include "magick/resource_.h"
+#include "magick/option.h"
+#include "magick/pixel-accessor.h"
+#include "magick/property.h"
 #include "magick/quantum-private.h"
+#include "magick/resource_.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
 #include "magick/module.h"
+#include "magick/token.h"
+#include "magick/utility.h"
 
 /*
   Forward declarations.
@@ -88,10 +94,10 @@ static MagickBooleanType
 %
 %  The format of the RegisterHISTOGRAMImage method is:
 %
-%      unsigned long RegisterHISTOGRAMImage(void)
+%      size_t RegisterHISTOGRAMImage(void)
 %
 */
-ModuleExport unsigned long RegisterHISTOGRAMImage(void)
+ModuleExport size_t RegisterHISTOGRAMImage(void)
 {
   MagickInfo
     *entry;
@@ -99,9 +105,9 @@ ModuleExport unsigned long RegisterHISTOGRAMImage(void)
   entry=SetMagickInfo("HISTOGRAM");
   entry->encoder=(EncodeImageHandler *) WriteHISTOGRAMImage;
   entry->adjoin=MagickFalse;
-  entry->format_type=ExplicitFormatType;
+  entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Histogram of the image");
-  entry->module=ConstantString("HISTOGRAM");
+  entry->magick_module=ConstantString("HISTOGRAM");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -163,14 +169,6 @@ ModuleExport void UnregisterHISTOGRAMImage(void)
 %    o image:  The image.
 %
 */
-
-static inline size_t MagickMax(const size_t x,const size_t y)
-{
-  if (x > y)
-    return(x);
-  return(y);
-}
-
 static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   Image *image)
 {
@@ -182,24 +180,21 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   char
     filename[MaxTextExtent];
 
+  const char
+    *option;
+
+  const MagickInfo
+    *magick_info;
+
   ExceptionInfo
     *exception;
 
-  FILE
-    *file;
-
   Image
     *histogram_image;
 
   ImageInfo
     *write_info;
 
-  int
-    unique_file;
-
-  long
-    y;
-
   MagickBooleanType
     status;
 
@@ -213,27 +208,30 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   RectangleInfo
     geometry;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register long
-    x;
-
-  register PixelPacket
+  PixelPacket
     *q,
     *r;
 
+  ssize_t
+    x;
+
   size_t
     length;
 
+  ssize_t
+    y;
+
   /*
     Allocate histogram image.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
+  assert(image->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   SetGeometry(image,&geometry);
@@ -249,7 +247,7 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   /*
     Allocate histogram count arrays.
   */
-  length=MagickMax((size_t) ScaleQuantumToChar((Quantum) QuantumRange)+1UL,
+  length=MagickMax((size_t) ScaleQuantumToChar(QuantumRange)+1UL,
     histogram_image->columns);
   histogram=(MagickPixelPacket *) AcquireQuantumMemory(length,
     sizeof(*histogram));
@@ -262,25 +260,25 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
     Initialize histogram count arrays.
   */
   channel=image_info->channel;
-  (void) ResetMagickMemory(histogram,0,length*sizeof(*histogram));
-  for (y=0; y < (long) image->rows; y++)
+  (void) memset(histogram,0,length*sizeof(*histogram));
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       if ((channel & RedChannel) != 0)
-        histogram[ScaleQuantumToChar(p->red)].red++;
+        histogram[ScaleQuantumToChar(GetPixelRed(p))].red++;
       if ((channel & GreenChannel) != 0)
-        histogram[ScaleQuantumToChar(p->green)].green++;
+        histogram[ScaleQuantumToChar(GetPixelGreen(p))].green++;
       if ((channel & BlueChannel) != 0)
-        histogram[ScaleQuantumToChar(p->blue)].blue++;
+        histogram[ScaleQuantumToChar(GetPixelBlue(p))].blue++;
       p++;
     }
   }
   maximum=histogram[0].red;
-  for (x=0; x < (long) histogram_image->columns; x++)
+  for (x=0; x < (ssize_t) histogram_image->columns; x++)
   {
     if (((channel & RedChannel) != 0) && (maximum < histogram[x].red))
       maximum=histogram[x].red;
@@ -289,52 +287,58 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
     if (((channel & BlueChannel) != 0) && (maximum < histogram[x].blue))
       maximum=histogram[x].blue;
   }
-  scale=(MagickRealType) histogram_image->rows/maximum;
+  scale=0.0;
+  if (fabs((double) maximum) >= MagickEpsilon)
+    scale=(MagickRealType) histogram_image->rows/maximum;
   /*
     Initialize histogram image.
   */
   exception=(&image->exception);
-  (void) QueryColorDatabase("#000000",&histogram_image->background_color,
+  (void) QueryColorDatabase("#000",&histogram_image->background_color,
     &image->exception);
   (void) SetImageBackgroundColor(histogram_image);
-  for (x=0; x < (long) histogram_image->columns; x++)
+  for (x=0; x < (ssize_t) histogram_image->columns; x++)
   {
     q=GetAuthenticPixels(histogram_image,x,0,1,histogram_image->rows,exception);
     if (q == (PixelPacket *) NULL)
       break;
     if ((channel & RedChannel) != 0)
       {
-        y=(long) (histogram_image->rows-scale*histogram[x].red+0.5);
+        y=CastDoubleToLong(ceil((double) histogram_image->rows-scale*
+          histogram[x].red-0.5));
         r=q+y;
-        for ( ; y < (long) histogram_image->rows; y++)
+        for ( ; y < (ssize_t) histogram_image->rows; y++)
         {
-          r->red=(Quantum) QuantumRange;
+          SetPixelRed(r,QuantumRange);
           r++;
         }
       }
     if ((channel & GreenChannel) != 0)
       {
-        y=(long) (histogram_image->rows-scale*histogram[x].green+0.5);
+        y=CastDoubleToLong(ceil((double) histogram_image->rows-scale*
+          histogram[x].green-0.5));
         r=q+y;
-        for ( ; y < (long) histogram_image->rows; y++)
+        for ( ; y < (ssize_t) histogram_image->rows; y++)
         {
-          r->green=(Quantum) QuantumRange;
+          SetPixelGreen(r,QuantumRange);
           r++;
         }
       }
     if ((channel & BlueChannel) != 0)
       {
-        y=(long) (histogram_image->rows-scale*histogram[x].blue+0.5);
+        y=CastDoubleToLong(ceil((double) histogram_image->rows-scale*
+          histogram[x].blue-0.5));
         r=q+y;
-        for ( ; y < (long) histogram_image->rows; y++)
+        for ( ; y < (ssize_t) histogram_image->rows; y++)
         {
-          r->blue=(Quantum) QuantumRange;
+          SetPixelBlue(r,QuantumRange);
           r++;
         }
       }
     if (SyncAuthenticPixels(histogram_image,exception) == MagickFalse)
       break;
-    status=SetImageProgress(image,SaveImageTag,y,histogram_image->rows);
+    status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+      histogram_image->rows);
     if (status == MagickFalse)
       break;
   }
@@ -342,37 +346,53 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
     Relinquish resources.
   */
   histogram=(MagickPixelPacket *) RelinquishMagickMemory(histogram);
-  file=(FILE *) NULL;
-  unique_file=AcquireUniqueFileResource(filename);
-  if (unique_file != -1)
-    file=fdopen(unique_file,"wb");
-  if ((unique_file != -1) && (file != (FILE *) NULL))
+  option=GetImageOption(image_info,"histogram:unique-colors");
+  if ((IsHistogramImage(image,exception) != MagickFalse) ||
+      (IsMagickTrue(option) != MagickFalse) ||
+      (GetImageOption(image_info,"format") != (const char *) NULL))
     {
-      char
-        *property;
+      FILE
+        *file;
+
+      int
+        unique_file;
 
       /*
-        Add a histogram as an image comment.
+        Add a unique colors as an image comment.
       */
-      (void) GetNumberColors(image,file,&image->exception);
-      (void) fclose(file);
-      property=FileToString(filename,~0UL,&image->exception);
-      if (property != (char *) NULL)
+      file=(FILE *) NULL;
+      unique_file=AcquireUniqueFileResource(filename);
+      if (unique_file != -1)
+        file=fdopen(unique_file,"wb");
+      if ((unique_file != -1) && (file != (FILE *) NULL))
         {
-          (void) SetImageProperty(histogram_image,"comment",property);
-          property=DestroyString(property);
+          char
+            *property;
+
+          (void) GetNumberColors(image,file,&image->exception);
+          (void) fclose(file);
+          property=FileToString(filename,~0UL,&image->exception);
+          if (property != (char *) NULL)
+            {
+              (void) SetImageProperty(histogram_image,"comment",property);
+              property=DestroyString(property);
+            }
         }
+      (void) RelinquishUniqueFileResource(filename);
     }
-  (void) RelinquishUniqueFileResource(filename);
   /*
     Write Histogram image.
   */
   (void) CopyMagickString(histogram_image->filename,image_info->filename,
     MaxTextExtent);
+  (void) ResetImagePage(histogram_image,"0x0+0+0");
   write_info=CloneImageInfo(image_info);
-  (void) SetImageInfo(write_info,MagickTrue,&image->exception);
-  if (LocaleCompare(write_info->magick,"HISTOGRAM") == 0)
-    (void) FormatMagickString(histogram_image->filename,MaxTextExtent,
+  *write_info->magick='\0';
+  (void) SetImageInfo(write_info,1,&image->exception);
+  magick_info=GetMagickInfo(write_info->magick,&image->exception);
+  if ((magick_info == (const MagickInfo*) NULL) ||
+      (LocaleCompare(magick_info->magick_module,"HISTOGRAM") == 0))
+    (void) FormatLocaleString(histogram_image->filename,MaxTextExtent,
       "miff:%s",write_info->filename);
   status=WriteImage(write_info,histogram_image);
   histogram_image=DestroyImage(histogram_image);
