@@ -17,13 +17,13 @@
 %                            Christopher R. Hawks                             %
 %                               December 2001                                 %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
-%   obtain a copy of the License at                                            %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -42,14 +42,19 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/cache.h"
 #include "magick/color.h"
+#include "magick/colormap.h"
+#include "magick/colormap-private.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
+#include "magick/colorspace-private.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
+#include "magick/histogram.h"
 #include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
@@ -57,6 +62,7 @@
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
 #include "magick/paint.h"
+#include "magick/pixel-accessor.h"
 #include "magick/pixel-private.h"
 #include "magick/property.h"
 #include "magick/quantize.h"
@@ -83,7 +89,7 @@
 /*
  The 256 color system palette for Palm Computing Devices.
 */
-static unsigned char
+static const unsigned char
   PalmPalette[256][3] =
   {
     {255, 255,255}, {255, 204,255}, {255, 153,255}, {255, 102,255},
@@ -183,15 +189,15 @@ static MagickBooleanType
 %    o pixel: a pointer to the PixelPacket to be matched.
 %
 */
-static int FindColor(PixelPacket *pixel)
+static ssize_t FindColor(PixelPacket *pixel)
 {
-  register long
+  ssize_t
     i;
 
   for (i=0; i < 256; i++)
-    if (ScaleQuantumToChar(pixel->red) == PalmPalette[i][0] &&
-        ScaleQuantumToChar(pixel->green) == PalmPalette[i][1] &&
-        ScaleQuantumToChar(pixel->blue) == PalmPalette[i][2])
+    if (ScaleQuantumToChar(GetPixelRed(pixel)) == PalmPalette[i][0] &&
+        ScaleQuantumToChar(GetPixelGreen(pixel)) == PalmPalette[i][1] &&
+        ScaleQuantumToChar(GetPixelBlue(pixel)) == PalmPalette[i][2])
       return(i);
   return(-1);
 }
@@ -223,61 +229,39 @@ static int FindColor(PixelPacket *pixel)
 %    o exception: return any errors or warnings in this structure.
 %
 */
-
-static inline long MagickMax(const long x,const long y)
-{
-  if (x > y)
-    return(x);
-  return(y);
-}
-
-static inline ssize_t MagickMin(const ssize_t x,const ssize_t y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
 static Image *ReadPALMImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
+  const size_t
+    one = 1;
+
   Image
     *image;
 
   IndexPacket
     index;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    totalOffset,
+    seekNextDepth;
+
   MagickPixelPacket
     transpix;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register long
-    i,
-    x;
-
-  register PixelPacket
+  PixelPacket
     *q;
 
-  ssize_t
-    count;
-
-  unsigned char
-    *lastrow,
-    *one_row,
-    *ptr;
-
-  unsigned long
-    bytes_per_row,
-    flags,
+  size_t
     bits_per_pixel,
+    bytes_per_row,
+    extent,
+    flags,
     version,
     nextDepthOffset,
     transparentIndex,
@@ -291,23 +275,30 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
     size,
     bit;
 
+  ssize_t
+    i,
+    count,
+    x,
+    y;
+
+  unsigned char
+    *last_row,
+    *one_row,
+    *ptr;
+
   unsigned short
     color16;
 
-  MagickOffsetType
-    totalOffset,
-    seekNextDepth;
-
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
-  if (image_info->debug != MagickFalse)
+  assert(image_info->signature == MagickCoreSignature);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
   image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -322,33 +313,49 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
     image->rows=ReadBlobMSBShort(image);
     if (EOFBlob(image) != MagickFalse)
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-    /*
-      Copied from coders/pnm.c. TODO other checks ...
-    */
     if ((image->columns == 0) || (image->rows == 0))
       ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
+    status=SetImageExtent(image,image->columns,image->rows);
+    if (status == MagickFalse)
+      {
+        InheritException(exception,&image->exception);
+        return(DestroyImageList(image));
+      }
+    (void) SetImageBackgroundColor(image);
     bytes_per_row=ReadBlobMSBShort(image);
     flags=ReadBlobMSBShort(image);
-    bits_per_pixel=(unsigned long) ReadBlobByte(image);
-    if (bits_per_pixel > 16)
-      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-    version=(unsigned long) ReadBlobByte(image);
-    nextDepthOffset=(unsigned long) ReadBlobMSBShort(image);
-    transparentIndex=(unsigned long) ReadBlobByte(image);
-    compressionType=(unsigned long) ReadBlobByte(image);
+    bits_per_pixel=(size_t) ReadBlobByte(image);
+    if ((bits_per_pixel != 1) && (bits_per_pixel != 2) &&
+        (bits_per_pixel != 4) && (bits_per_pixel != 8) &&
+        (bits_per_pixel != 16))
+      ThrowReaderException(CorruptImageError,"UnsupportedBitsPerPixel");
+    version=(size_t) ReadBlobByte(image);
+    if ((version != 0) && (version != 1) && (version != 2))
+      ThrowReaderException(CorruptImageError,"FileFormatVersionMismatch");
+    nextDepthOffset=(size_t) ReadBlobMSBShort(image);
+    transparentIndex=(size_t) ReadBlobByte(image);
+    compressionType=(size_t) ReadBlobByte(image);
+    if ((compressionType != PALM_COMPRESSION_NONE) &&
+        (compressionType != PALM_COMPRESSION_SCANLINE ) &&
+        (compressionType != PALM_COMPRESSION_RLE))
+      ThrowReaderException(CorruptImageError,"UnrecognizedImageCompression");
     pad=ReadBlobMSBShort(image);
+    (void) pad;
     /*
       Initialize image colormap.
     */
     if ((bits_per_pixel < 16) &&
-        (AcquireImageColormap(image,1L << bits_per_pixel) == MagickFalse))
+        (AcquireImageColormap(image,one << bits_per_pixel) == MagickFalse))
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     GetMagickPixelPacket(image,&transpix);
     if (bits_per_pixel == 16)  /* Direct Color */
       {
-        redbits=(unsigned long) ReadBlobByte(image);  /* # of bits of red */
-        greenbits=(unsigned long) ReadBlobByte(image);  /* # of bits of green */
-        bluebits=(unsigned long) ReadBlobByte(image);  /* # of bits of blue */
+        redbits=(size_t) ReadBlobByte(image);  /* # of bits of red */
+        (void) redbits;
+        greenbits=(size_t) ReadBlobByte(image);  /* # of bits of green */
+        (void) greenbits;
+        bluebits=(size_t) ReadBlobByte(image);  /* # of bits of blue */
+        (void) bluebits;
         ReadBlobByte(image);  /* reserved by Palm */
         ReadBlobByte(image);  /* reserved by Palm */
         transpix.red=(MagickRealType) (QuantumRange*ReadBlobByte(image)/31);
@@ -363,98 +370,120 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
         if (flags & PALM_HAS_COLORMAP_FLAG)
           {
             count=(ssize_t) ReadBlobMSBShort(image);
-            for (i=0; i < (long) count; i++)
+            for (i=0; i < (ssize_t) count; i++)
             {
               ReadBlobByte(image);
-              index=ConstrainColormapIndex(image,255-i);
-              image->colormap[(int) index].red=
-                ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-              image->colormap[(int) index].green=
-                ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-              image->colormap[(int) index].blue=
-                ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
+              index=ConstrainColormapIndex(image,(ssize_t) (255-i));
+              image->colormap[(int) index].red=ScaleCharToQuantum(
+                (unsigned char) ReadBlobByte(image));
+              image->colormap[(int) index].green=ScaleCharToQuantum(
+                (unsigned char) ReadBlobByte(image));
+              image->colormap[(int) index].blue=ScaleCharToQuantum(
+                (unsigned char) ReadBlobByte(image));
           }
         }
       else
+        for (i=0; i < (ssize_t) (1L << bits_per_pixel); i++)
         {
-          for (i=0; i < (long) (1L << bits_per_pixel); i++)
-          {
-            index=ConstrainColormapIndex(image,255-i);
-            image->colormap[(int) index].red=
-              ScaleCharToQuantum(PalmPalette[i][0]);
-            image->colormap[(int) index].green=
-              ScaleCharToQuantum(PalmPalette[i][1]);
-            image->colormap[(int) index].blue=
-              ScaleCharToQuantum(PalmPalette[i][2]);
-          }
+          index=ConstrainColormapIndex(image,(ssize_t) (255-i));
+          image->colormap[(int) index].red=ScaleCharToQuantum(
+            PalmPalette[i][0]);
+          image->colormap[(int) index].green=ScaleCharToQuantum(
+            PalmPalette[i][1]);
+          image->colormap[(int) index].blue=ScaleCharToQuantum(
+            PalmPalette[i][2]);
         }
       }
     if (flags & PALM_IS_COMPRESSED_FLAG)
       size=ReadBlobMSBShort(image);
+    (void) size;
     image->storage_class=DirectClass;
     if (bits_per_pixel < 16)
       {
         image->storage_class=PseudoClass;
         image->depth=8;
       }
-    if (SetImageExtent(image,0,0) == MagickFalse)
+    if (image_info->ping != MagickFalse)
+      {
+        (void) CloseBlob(image);
+        return(image);
+      }
+    status=SetImageExtent(image,image->columns,image->rows);
+    if (status == MagickFalse)
       {
         InheritException(exception,&image->exception);
         return(DestroyImageList(image));
       }
-    one_row=(unsigned char *) AcquireQuantumMemory(bytes_per_row,
-      sizeof(*one_row));
+    extent=MagickMax(bytes_per_row,2*image->columns);
+    one_row=(unsigned char *) AcquireQuantumMemory(extent,sizeof(*one_row));
     if (one_row == (unsigned char *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    lastrow=(unsigned char *) NULL;
-    if (compressionType == PALM_COMPRESSION_SCANLINE) {
-      lastrow=(unsigned char *) AcquireQuantumMemory(bytes_per_row,
-        sizeof(*lastrow));
-    if (lastrow == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    }
-    mask=(1l << bits_per_pixel)-1;
-    for (y = 0; y < (long) image->rows; y++)
+    (void) memset(one_row,0,extent*sizeof(*one_row));
+    last_row=(unsigned char *) NULL;
+    if (compressionType == PALM_COMPRESSION_SCANLINE)
+      {
+        last_row=(unsigned char *) AcquireQuantumMemory(MagickMax(bytes_per_row,
+          2*image->columns),sizeof(*last_row));
+        if (last_row == (unsigned char *) NULL)
+          {
+            one_row=(unsigned char *) RelinquishMagickMemory(one_row);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+          }
+         (void) memset(last_row,0,MagickMax(bytes_per_row,2*image->columns)*
+           sizeof(*last_row));
+      }
+    mask=(size_t) (1U << bits_per_pixel)-1;
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
       if ((flags & PALM_IS_COMPRESSED_FLAG) == 0)
         {
           /* TODO move out of loop! */
           image->compression=NoCompression;
           count=ReadBlob(image,bytes_per_row,one_row);
+          if (count != (ssize_t) bytes_per_row)
+            break;
         }
       else
         {
           if (compressionType == PALM_COMPRESSION_RLE)
-            { 
+            {
               /* TODO move out of loop! */
               image->compression=RLECompression;
-              for (i=0; i < (long) bytes_per_row; )
+              for (i=0; i < (ssize_t) bytes_per_row; )
               {
                 count=(ssize_t) ReadBlobByte(image);
+                if (count < 0)
+                  break;
                 count=MagickMin(count,(ssize_t) bytes_per_row-i);
-                byte=(unsigned long) ReadBlobByte(image);
-                (void) ResetMagickMemory(one_row+i,(int) byte,(size_t) count);
+                byte=(size_t) ReadBlobByte(image);
+                (void) memset(one_row+i,(int) byte,(size_t) count);
                 i+=count;
               }
           }
         else
           if (compressionType == PALM_COMPRESSION_SCANLINE)
-            {  
+            {
+              size_t
+                one;
+
               /* TODO move out of loop! */
+              one=1;
               image->compression=FaxCompression;
-              for (i=0; i < (long) bytes_per_row; i+=8)
+              for (i=0; i < (ssize_t) bytes_per_row; i+=8)
               {
                 count=(ssize_t) ReadBlobByte(image);
-                byte=1UL*MagickMin((ssize_t) bytes_per_row-i,8);
+                if (count < 0)
+                  break;
+                byte=(size_t) MagickMin((ssize_t) bytes_per_row-i,8);
                 for (bit=0; bit < byte; bit++)
                 {
-                  if ((y == 0) || (count & (1 << (7 - bit))))
+                  if ((y == 0) || (count & (one << (7 - bit))))
                     one_row[i+bit]=(unsigned char) ReadBlobByte(image);
                   else
-                    one_row[i+bit]=lastrow[i+bit];
+                    one_row[i+bit]=last_row[i+bit];
                 }
               }
-              (void) CopyMagickMemory(lastrow, one_row, bytes_per_row);
+              (void) memcpy(last_row, one_row, bytes_per_row);
             }
         }
       ptr=one_row;
@@ -465,28 +494,38 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
       if (bits_per_pixel == 16)
         {
           if (image->columns > (2*bytes_per_row))
-            ThrowReaderException(CorruptImageError,"CorruptImage");
-          for (x=0; x < (long) image->columns; x++)
+            {
+              one_row=(unsigned char *) RelinquishMagickMemory(one_row);
+              if (compressionType == PALM_COMPRESSION_SCANLINE)
+                last_row=(unsigned char *) RelinquishMagickMemory(last_row);
+              ThrowReaderException(CorruptImageError,"CorruptImage");
+            }
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
             color16=(*ptr++ << 8);
             color16|=(*ptr++);
-            q->red=(Quantum) ((QuantumRange*((color16 >> 11) & 0x1f))/0x1f);
-            q->green=(Quantum) ((QuantumRange*((color16 >> 5) & 0x3f))/0x3f);
-            q->blue=(Quantum) ((QuantumRange*((color16 >> 0) & 0x1f))/0x1f);
-            q->opacity=OpaqueOpacity;
+            SetPixelRed(q,(QuantumRange*((color16 >> 11) & 0x1f))/0x1f);
+            SetPixelGreen(q,(QuantumRange*((color16 >> 5) & 0x3f))/0x3f);
+            SetPixelBlue(q,(QuantumRange*((color16 >> 0) & 0x1f))/0x1f);
+            SetPixelOpacity(q,OpaqueOpacity);
             q++;
           }
         }
       else
         {
           bit=8-bits_per_pixel;
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
             if ((size_t) (ptr-one_row) >= bytes_per_row)
-              ThrowReaderException(CorruptImageError,"CorruptImage");
+              {
+                one_row=(unsigned char *) RelinquishMagickMemory(one_row);
+                if (compressionType == PALM_COMPRESSION_SCANLINE)
+                  last_row=(unsigned char *) RelinquishMagickMemory(last_row);
+                ThrowReaderException(CorruptImageError,"CorruptImage");
+              }
             index=(IndexPacket) (mask-(((*ptr) & (mask << bit)) >> bit));
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetPixelIndex(indexes+x,index);
+            SetPixelRGBO(q,image->colormap+(ssize_t) index);
             if (bit)
               bit-=bits_per_pixel;
             else
@@ -494,28 +533,37 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
                 ptr++;
                 bit=8-bits_per_pixel;
               }
+            q++;
           }
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
         }
         if (image->previous == (Image *) NULL)
           {
-            status=SetImageProgress(image,LoadImageTag,y,image->rows);
+            status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+              image->rows);
             if (status == MagickFalse)
               break;
           }
       }
     if (flags & PALM_HAS_TRANSPARENCY_FLAG)
       {
+        IndexPacket index=ConstrainColormapIndex(image,(mask-transparentIndex));
         if (bits_per_pixel != 16)
-          SetMagickPixelPacket(image,image->colormap+(mask-transparentIndex),
+          SetMagickPixelPacket(image,image->colormap+(ssize_t) index,
             (const IndexPacket *) NULL,&transpix);
         (void) TransparentPaintImage(image,&transpix,(Quantum)
           TransparentOpacity,MagickFalse);
       }
     one_row=(unsigned char *) RelinquishMagickMemory(one_row);
     if (compressionType == PALM_COMPRESSION_SCANLINE)
-    lastrow=(unsigned char *) RelinquishMagickMemory(lastrow);
+      last_row=(unsigned char *) RelinquishMagickMemory(last_row);
+    if (EOFBlob(image) != MagickFalse)
+      {
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image. Copied from coders/pnm.c
     */
@@ -523,19 +571,15 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     if (nextDepthOffset != 0)
-      { 
+      {
         /*
-          Skip to next image. 
+          Skip to next image.
         */
         totalOffset+=(MagickOffsetType) (nextDepthOffset*4);
         if (totalOffset >= (MagickOffsetType) GetBlobSize(image))
-          {
-            ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-          }
+          ThrowReaderException(CorruptImageError,"ImproperImageHeader")
         else
-          {
-            seekNextDepth=SeekBlob(image,totalOffset,SEEK_SET);
-          }
+          seekNextDepth=SeekBlob(image,totalOffset,SEEK_SET);
         if (seekNextDepth != totalOffset)
           ThrowReaderException(CorruptImageError,"ImproperImageHeader");
         /*
@@ -544,8 +588,8 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
         AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
-            (void) DestroyImageList(image);
-            return((Image *) NULL);
+            status=MagickFalse;
+            break;
           }
         image=SyncNextImageInList(image);
         status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
@@ -554,7 +598,10 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
           break;
       }
   } while (nextDepthOffset != 0);
-  (void) CloseBlob(image);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
 
@@ -577,10 +624,10 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterPALMImage method is:
 %
-%      unsigned long RegisterPALMImage(void)
+%      size_t RegisterPALMImage(void)
 %
 */
-ModuleExport unsigned long RegisterPALMImage(void)
+ModuleExport size_t RegisterPALMImage(void)
 {
   MagickInfo
     *entry;
@@ -590,7 +637,7 @@ ModuleExport unsigned long RegisterPALMImage(void)
   entry->encoder=(EncodeImageHandler *) WritePALMImage;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Palm pixmap");
-  entry->module=ConstantString("PALM");
+  entry->magick_module=ConstantString("PALM");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -648,10 +695,13 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
   Image *image)
 {
   int
-    y;
+    bit;
 
   ExceptionInfo
-    exception;
+    *exception;
+
+  IndexPacket
+    *indexes;
 
   MagickBooleanType
     status;
@@ -670,20 +720,25 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
   QuantizeInfo
     *quantize_info;
 
-  register IndexPacket
-    *indexes;
-
-  register long
-    x;
-
-  register PixelPacket
+  PixelPacket
     *p;
 
+  size_t
+    count,
+    bits_per_pixel,
+    bytes_per_row,
+    nextDepthOffset,
+    number_scenes,
+    one;
+
+  ssize_t
+    x,
+    y;
+
   unsigned char
-    bit,
     byte,
     color,
-    *lastrow,
+    *last_row,
     *one_row,
     *ptr,
     version;
@@ -691,30 +746,23 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
   unsigned int
     transparentIndex;
 
-  unsigned long
-    count,
-    bits_per_pixel,
-    bytes_per_row,
-    nextDepthOffset;
-
   unsigned short
     color16,
     flags;
 
-
   /*
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
+  assert(image->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&exception);
+  exception=AcquireExceptionInfo();
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
-  GetExceptionInfo(&exception);
   quantize_info=AcquireQuantizeInfo(image_info);
   flags=0;
   currentOffset=0;
@@ -723,21 +771,24 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
   transpix.green=0;
   transpix.blue=0;
   transpix.opacity=0;
+  one=1;
   version=0;
   scene=0;
+  number_scenes=GetImageListLength(image);
   do
   {
-    if (image->colorspace != RGBColorspace)
-      (void) SetImageColorspace(image,RGBColorspace);
-    count=GetNumberColors(image,NULL,&exception);
-    for (bits_per_pixel=1;  (1UL << bits_per_pixel) < count;  bits_per_pixel*=2) ;
-    if (image_info->depth > 100)
-      bits_per_pixel=image_info->depth-100;
-    if (bits_per_pixel < 16)
-      (void) SetImageColorspace(image,image->colorspace);
+    if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+      (void) TransformImageColorspace(image,sRGBColorspace);
+    count=GetNumberColors(image,NULL,exception);
+    for (bits_per_pixel=1; (bits_per_pixel < 64) && (one << bits_per_pixel) < count; bits_per_pixel*=2) ;
+    if (bits_per_pixel > 16)
+      bits_per_pixel=16;
+    else
+      if (bits_per_pixel < 16)
+        (void) TransformImageColorspace(image,image->colorspace);
     if (bits_per_pixel < 8)
       {
-        (void) SetImageColorspace(image,GRAYColorspace);
+        (void) TransformImageColorspace(image,GRAYColorspace);
         (void) SetImageType(image,PaletteType);
         (void) SortColormapByIntensity(image);
       }
@@ -809,53 +860,65 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
             *affinity_image;
 
           affinity_image=ConstituteImage(256,1,"RGB",CharPixel,&PalmPalette,
-            &exception);
-          (void) SetImageColorspace(affinity_image,affinity_image->colorspace);
+            exception);
+          (void) TransformImageColorspace(affinity_image,
+            affinity_image->colorspace);
           (void) RemapImage(quantize_info,image,affinity_image);
-          for (y=0; y < (long) image->rows; y++)
+          for (y=0; y < (ssize_t) image->rows; y++)
           {
-            p=GetAuthenticPixels(image,0,y,image->columns,1,&exception);
+            p=GetAuthenticPixels(image,0,y,image->columns,1,exception);
             indexes=GetAuthenticIndexQueue(image);
-            for (x=0; x < (long) image->columns; x++)
-              indexes[x]=(IndexPacket) FindColor(&image->colormap[(long)
-                indexes[x]]);
+            for (x=0; x < (ssize_t) image->columns; x++)
+              SetPixelIndex(indexes+x,FindColor(&image->colormap[
+                (ssize_t) GetPixelIndex(indexes+x)]));
           }
           affinity_image=DestroyImage(affinity_image);
         }
       }
     if (flags & PALM_IS_COMPRESSED_FLAG)
       (void) WriteBlobMSBShort(image,0);  /* fill in size later */
-    lastrow=(unsigned char *) NULL;
+    last_row=(unsigned char *) NULL;
     if (image_info->compression == FaxCompression)
-      lastrow=(unsigned char *) AcquireQuantumMemory(bytes_per_row,
-        sizeof(*lastrow));
-      /* TODO check whether memory really was acquired? */
-    one_row=(unsigned char *) AcquireQuantumMemory(bytes_per_row,
+      {
+        last_row=(unsigned char *) AcquireQuantumMemory(bytes_per_row+256,
+          sizeof(*last_row));
+        if (last_row == (unsigned char *) NULL)
+          {
+            quantize_info=DestroyQuantizeInfo(quantize_info);
+            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+          }
+      }
+    one_row=(unsigned char *) AcquireQuantumMemory(bytes_per_row+256,
       sizeof(*one_row));
     if (one_row == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-    for (y=0; y < (int) image->rows; y++)
+      {
+        if (last_row != (unsigned char *) NULL)
+          last_row=(unsigned char *) RelinquishMagickMemory(last_row);
+        quantize_info=DestroyQuantizeInfo(quantize_info);
+        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+      }
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
       ptr=one_row;
-      (void) ResetMagickMemory(ptr,0,bytes_per_row);
-      p=GetAuthenticPixels(image,0,y,image->columns,1,&exception);
+      (void) memset(ptr,0,bytes_per_row);
+      p=GetAuthenticPixels(image,0,y,image->columns,1,exception);
       if (p == (PixelPacket *) NULL)
         break;
       indexes=GetAuthenticIndexQueue(image);
       if (bits_per_pixel == 16)
         {
-          for (x=0; x < (int) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
-            color16=(unsigned short) ((((31*(unsigned long) p->red)/
-              (unsigned long) QuantumRange) << 11) |
-              (((63*(unsigned long) p->green)/(unsigned long) QuantumRange) << 5) |
-              ((31*(unsigned long) p->blue)/(unsigned long) QuantumRange));
-            if (p->opacity == (Quantum) TransparentOpacity)
+            color16=(unsigned short) ((((31*(ssize_t) GetPixelRed(p))/
+              (ssize_t) QuantumRange) << 11) |
+              (((63*(ssize_t) GetPixelGreen(p))/(ssize_t) QuantumRange) << 5) |
+              ((31*(ssize_t) GetPixelBlue(p))/(ssize_t) QuantumRange));
+            if (GetPixelOpacity(p) == (Quantum) TransparentOpacity)
               {
-                transpix.red=p->red;
-                transpix.green=p->green;
-                transpix.blue=p->blue;
-                transpix.opacity=p->opacity;
+                transpix.red=GetPixelRed(p);
+                transpix.green=GetPixelGreen(p);
+                transpix.blue=GetPixelBlue(p);
+                transpix.opacity=GetPixelOpacity(p);
                 flags|=PALM_HAS_TRANSPARENCY_FLAG;
               }
             *ptr++=(unsigned char) ((color16 >> 8) & 0xff);
@@ -867,16 +930,16 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
         {
           byte=0x00;
           bit=(unsigned char) (8-bits_per_pixel);
-          for (x=0; x < (int) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
             if (bits_per_pixel >= 8)
-              color=(unsigned char) indexes[x];
+              color=(unsigned char) ((size_t) GetPixelIndex(indexes+x));
             else
-              color=(unsigned char) (indexes[x]*((1 << bits_per_pixel)-1)/
-                MagickMax(1L*image->colors-1L,1L));
+              color=(unsigned char) (GetPixelIndex(indexes+x)*
+                ((one << bits_per_pixel)-1)/MagickMax(1*image->colors-1,1));
             byte|=color << bit;
             if (bit != 0)
-              bit-=bits_per_pixel;
+              bit-=(unsigned char) bits_per_pixel;
             else
               {
                 *ptr++=byte;
@@ -890,12 +953,12 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
       if (image_info->compression == RLECompression)
         {
           x=0;
-          while (x < (long) bytes_per_row)
+          while (x < (ssize_t) bytes_per_row)
           {
             byte=one_row[x];
             count=1;
             while ((one_row[++x] == byte) && (count < 255) &&
-                   (x < (long) bytes_per_row))
+                   (x < (ssize_t) bytes_per_row))
               count++;
             (void) WriteBlobByte(image,(unsigned char) count);
             (void) WriteBlobByte(image,(unsigned char) byte);
@@ -907,22 +970,22 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
             char
               tmpbuf[8],
               *tptr;
-  
-            for (x = 0;  x < (long) bytes_per_row;  x += 8)
+
+            for (x=0;  x < (ssize_t) bytes_per_row;  x+=8)
             {
               tptr = tmpbuf;
-              for (bit=0, byte=0; bit < (unsigned char) MagickMin(8,(ssize_t) bytes_per_row-x); bit++)
+              for (bit=0, byte=0; bit < (int) MagickMin(8L,(ssize_t) bytes_per_row-x); bit++)
               {
-                if ((y == 0) || (lastrow[x + bit] != one_row[x + bit]))
+                if ((y == 0) || (last_row[x + bit] != one_row[x + bit]))
                   {
-                  byte |= (1 << (7 - bit));
-                  *tptr++ = (char) one_row[x + bit];
-                }
+                    byte |= (1 << (7 - bit));
+                    *tptr++ = (char) one_row[x + bit];
+                  }
               }
               (void) WriteBlobByte(image, byte);
               (void) WriteBlob(image,tptr-tmpbuf,(unsigned char *) tmpbuf);
             }
-            (void) CopyMagickMemory(lastrow,one_row,bytes_per_row);
+            (void) memcpy(last_row,one_row,bytes_per_row);
           }
         else
           (void) WriteBlob(image,bytes_per_row,one_row);
@@ -938,9 +1001,12 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
       {
         offset=SeekBlob(image,currentOffset+20,SEEK_SET);
         (void) WriteBlobByte(image,0);  /* reserved by Palm */
-        (void) WriteBlobByte(image,(unsigned char) ((31*transpix.red)/QuantumRange));
-        (void) WriteBlobByte(image,(unsigned char) ((63*transpix.green)/QuantumRange));
-        (void) WriteBlobByte(image,(unsigned char) ((31*transpix.blue)/QuantumRange));
+        (void) WriteBlobByte(image,(unsigned char) ((31*transpix.red)/
+          QuantumRange));
+        (void) WriteBlobByte(image,(unsigned char) ((63*transpix.green)/
+          QuantumRange));
+        (void) WriteBlobByte(image,(unsigned char) ((31*transpix.blue)/
+          QuantumRange));
       }
     if (flags & PALM_IS_COMPRESSED_FLAG)  /* fill in size now */
       {
@@ -948,31 +1014,28 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
         (void) WriteBlobMSBShort(image,(unsigned short) (GetBlobSize(image)-
           currentOffset-offset));
       }
-    if (one_row != (unsigned char *) NULL) 
+    if (one_row != (unsigned char *) NULL)
       one_row=(unsigned char *) RelinquishMagickMemory(one_row);
-    if (lastrow != (unsigned char *) NULL) 
-      lastrow=(unsigned char *) RelinquishMagickMemory(lastrow);
+    if (last_row != (unsigned char *) NULL)
+      last_row=(unsigned char *) RelinquishMagickMemory(last_row);
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     /* padding to 4 byte word */
-    for (cc = (GetBlobSize(image))%4; cc > 0; cc--)
-    {
+    for (cc=(GetBlobSize(image)) % 4; cc > 0; cc--)
       (void) WriteBlobByte(image,0);
-    } 
     /* write nextDepthOffset and return to end of image */
-    offset=SeekBlob(image,currentOffset+10,SEEK_SET);
-    nextDepthOffset=(unsigned long) ((GetBlobSize(image)-currentOffset)/4);
+    (void) SeekBlob(image,currentOffset+10,SEEK_SET);
+    nextDepthOffset=(size_t) ((GetBlobSize(image)-currentOffset)/4);
     (void) WriteBlobMSBShort(image,(unsigned short) nextDepthOffset);
     currentOffset=(MagickOffsetType) GetBlobSize(image);
-    offset=SeekBlob(image,currentOffset,SEEK_SET);
+    (void) SeekBlob(image,currentOffset,SEEK_SET);
     image=SyncNextImageInList(image);
-    status=SetImageProgress(image,SaveImagesTag,scene++,
-      GetImageListLength(image));
+    status=SetImageProgress(image,SaveImagesTag,scene++,number_scenes);
     if (status == MagickFalse)
       break;
   } while (image_info->adjoin != MagickFalse);
   quantize_info=DestroyQuantizeInfo(quantize_info);
   (void) CloseBlob(image);
-  (void) DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo(exception);
   return(MagickTrue);
 }
