@@ -12,17 +12,17 @@
 %             MagicCore Methods to Acquire / Destroy Quantum Pixels           %
 %                                                                             %
 %                             Software Design                                 %
-%                               John Cristy                                   %
+%                                  Cristy                                     %
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,13 +39,16 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/property.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/color-private.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/cache.h"
+#include "magick/cache-private.h"
+#include "magick/colorspace.h"
+#include "magick/colorspace-private.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
 #include "magick/geometry.h"
@@ -56,6 +59,7 @@
 #include "magick/option.h"
 #include "magick/pixel.h"
 #include "magick/pixel-private.h"
+#include "magick/property.h"
 #include "magick/quantum.h"
 #include "magick/quantum-private.h"
 #include "magick/resource_.h"
@@ -63,6 +67,8 @@
 #include "magick/statistic.h"
 #include "magick/stream.h"
 #include "magick/string_.h"
+#include "magick/string-private.h"
+#include "magick/thread-private.h"
 #include "magick/utility.h"
 
 /*
@@ -70,6 +76,12 @@
 */
 #define QuantumSignature  0xab
 
+/*
+  Forward declarations.
+*/
+static void
+  DestroyQuantumPixels(QuantumInfo *);
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -94,28 +106,26 @@
 %    o image: the image.
 %
 */
-
-static inline unsigned long MagickMax(const unsigned long x,
-  const unsigned long y)
-{
-  if (x > y)
-    return(x);
-  return(y);
-}
-
 MagickExport QuantumInfo *AcquireQuantumInfo(const ImageInfo *image_info,
   Image *image)
 {
+  MagickBooleanType
+    status;
+
   QuantumInfo
     *quantum_info;
 
   quantum_info=(QuantumInfo *) AcquireMagickMemory(sizeof(*quantum_info));
   if (quantum_info == (QuantumInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  quantum_info->signature=MagickSignature;
+  quantum_info->signature=MagickCoreSignature;
   GetQuantumInfo(image_info,quantum_info);
-  if (image != (const Image *) NULL)
-    (void) SetQuantumDepth(image,quantum_info,image->depth);
+  if (image == (const Image *) NULL)
+    return(quantum_info);
+  status=SetQuantumDepth(image,quantum_info,image->depth);
+  quantum_info->endian=image->endian;
+  if (status == MagickFalse)
+    quantum_info=DestroyQuantumInfo(quantum_info);
   return(quantum_info);
 }
 
@@ -124,6 +134,65 @@ MagickExport QuantumInfo *AcquireQuantumInfo(const ImageInfo *image_info,
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   A c q u i r e Q u a n t u m P i x e l s                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AcquireQuantumPixels() allocates the pixel staging area.
+%
+%  The format of the AcquireQuantumPixels method is:
+%
+%      MagickBooleanType AcquireQuantumPixels(QuantumInfo *quantum_info,
+%        const size_t extent)
+%
+%  A description of each parameter follows:
+%
+%    o quantum_info: the quantum info.
+%
+%    o extent: the quantum info.
+%
+*/
+static MagickBooleanType AcquireQuantumPixels(QuantumInfo *quantum_info,
+  const size_t extent)
+{
+  ssize_t
+    i;
+
+  assert(quantum_info != (QuantumInfo *) NULL);
+  assert(quantum_info->signature == MagickCoreSignature);
+  quantum_info->number_threads=(size_t) GetMagickResourceLimit(ThreadResource);
+  quantum_info->pixels=(MemoryInfo **) AcquireQuantumMemory(
+    quantum_info->number_threads,sizeof(*quantum_info->pixels));
+  if (quantum_info->pixels == (MemoryInfo **) NULL)
+    return(MagickFalse);
+  quantum_info->extent=extent;
+  (void) memset(quantum_info->pixels,0,quantum_info->number_threads*
+    sizeof(*quantum_info->pixels));
+  for (i=0; i < (ssize_t) quantum_info->number_threads; i++)
+  {
+    unsigned char
+      *pixels;
+
+    quantum_info->pixels[i]=AcquireVirtualMemory(extent+1,sizeof(*pixels));
+    if (quantum_info->pixels[i] == (MemoryInfo *) NULL)
+      {
+        DestroyQuantumPixels(quantum_info);
+        return(MagickFalse);
+      }
+    pixels=(unsigned char *)  GetVirtualMemoryBlob(quantum_info->pixels[i]);
+    (void) memset(pixels,0,(extent+1)*sizeof(*pixels));
+    pixels[extent]=QuantumSignature;
+  }
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   D e s t r o y Q u a n t u m I n f o                                       %
 %                                                                             %
 %                                                                             %
@@ -145,16 +214,12 @@ MagickExport QuantumInfo *AcquireQuantumInfo(const ImageInfo *image_info,
 MagickExport QuantumInfo *DestroyQuantumInfo(QuantumInfo *quantum_info)
 {
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
-  if (quantum_info->pixels != (unsigned char *) NULL)
-    {
-      assert(quantum_info->pixels[quantum_info->extent] == QuantumSignature);
-      quantum_info->pixels=(unsigned char *) RelinquishMagickMemory(
-        quantum_info->pixels);
-    }
+  assert(quantum_info->signature == MagickCoreSignature);
+  if (quantum_info->pixels != (MemoryInfo **) NULL)
+    DestroyQuantumPixels(quantum_info);
   if (quantum_info->semaphore != (SemaphoreInfo *) NULL)
     DestroySemaphoreInfo(&quantum_info->semaphore);
-  quantum_info->signature=(~MagickSignature);
+  quantum_info->signature=(~MagickCoreSignature);
   quantum_info=(QuantumInfo *) RelinquishMagickMemory(quantum_info);
   return(quantum_info);
 }
@@ -164,6 +229,60 @@ MagickExport QuantumInfo *DestroyQuantumInfo(QuantumInfo *quantum_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   D e s t r o y Q u a n t u m P i x e l s                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  DestroyQuantumPixels() destroys the quantum pixels.
+%
+%  The format of the DestroyQuantumPixels() method is:
+%
+%      void DestroyQuantumPixels(QuantumInfo *quantum_info)
+%
+%  A description of each parameter follows:
+%
+%    o quantum_info: the quantum info.
+%
+*/
+static void DestroyQuantumPixels(QuantumInfo *quantum_info)
+{
+  ssize_t
+    i;
+
+  ssize_t
+    extent;
+
+  assert(quantum_info != (QuantumInfo *) NULL);
+  assert(quantum_info->signature == MagickCoreSignature);
+  assert(quantum_info->pixels != (MemoryInfo **) NULL);
+  extent=(ssize_t) quantum_info->extent;
+  for (i=0; i < (ssize_t) quantum_info->number_threads; i++)
+    if (quantum_info->pixels[i] != (MemoryInfo *) NULL)
+      {
+#ifndef NDEBUG
+        unsigned char
+          *pixels;
+
+        /*
+          Did we overrun our quantum buffer?
+        */
+        pixels=(unsigned char *) GetVirtualMemoryBlob(quantum_info->pixels[i]);
+        assert(pixels[extent] == QuantumSignature);
+#endif
+        quantum_info->pixels[i]=RelinquishVirtualMemory(
+          quantum_info->pixels[i]);
+      }
+  quantum_info->pixels=(MemoryInfo **) RelinquishMagickMemory(
+    quantum_info->pixels);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e t Q u a n t u m E x t e n t                                           %
 %                                                                             %
 %                                                                             %
@@ -190,30 +309,91 @@ MagickExport QuantumInfo *DestroyQuantumInfo(QuantumInfo *quantum_info)
 MagickExport size_t GetQuantumExtent(const Image *image,
   const QuantumInfo *quantum_info,const QuantumType quantum_type)
 {
-  MagickSizeType
-    number_pixels;
-
   size_t
-    packet_size;
+    channels;
 
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
-  packet_size=1;
+  assert(quantum_info->signature == MagickCoreSignature);
+  channels=1;
   switch (quantum_type)
   {
-    case GrayAlphaQuantum: packet_size=2; break;
-    case IndexAlphaQuantum: packet_size=2; break;
-    case RGBQuantum: packet_size=3; break;
-    case RGBAQuantum: packet_size=4; break;
-    case RGBOQuantum: packet_size=4; break;
-    case CMYKQuantum: packet_size=4; break;
-    case CMYKAQuantum: packet_size=5; break;
+    case GrayAlphaQuantum: channels=2; break;
+    case IndexAlphaQuantum: channels=2; break;
+    case RGBQuantum: channels=3; break;
+    case BGRQuantum: channels=3; break;
+    case RGBAQuantum: channels=4; break;
+    case RGBOQuantum: channels=4; break;
+    case BGRAQuantum: channels=4; break;
+    case CMYKQuantum: channels=4; break;
+    case CMYKAQuantum: channels=5; break;
+    case CbYCrAQuantum: channels=4; break;
+    case CbYCrQuantum: channels=3; break;
+    case CbYCrYQuantum: channels=4; break;
     default: break;
   }
-  number_pixels=GetImageExtent(image);
   if (quantum_info->pack == MagickFalse)
-    return((size_t) (packet_size*number_pixels*((image->depth+7)/8)));
-  return((size_t) ((packet_size*number_pixels*image->depth+7)/8));
+    return((size_t) (channels*image->columns*((quantum_info->depth+7)/8))+
+      (quantum_info->pad*image->columns));
+  return((size_t) ((channels*image->columns*quantum_info->depth+7)/8)+
+    (quantum_info->pad*image->columns));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   G e t Q u a n t u m E n d i a n                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetQuantumEndian() returns the quantum endian of the image.
+%
+%  The endian of the GetQuantumEndian method is:
+%
+%      EndianType GetQuantumEndian(const QuantumInfo *quantum_info)
+%
+%  A description of each parameter follows:
+%
+%    o quantum_info: the quantum info.
+%
+*/
+MagickExport EndianType GetQuantumEndian(const QuantumInfo *quantum_info)
+{
+  assert(quantum_info != (QuantumInfo *) NULL);
+  assert(quantum_info->signature == MagickCoreSignature);
+  return(quantum_info->endian);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   G e t Q u a n t u m F o r m a t                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetQuantumFormat() returns the quantum format of the image.
+%
+%  The format of the GetQuantumFormat method is:
+%
+%      QuantumFormatType GetQuantumFormat(const QuantumInfo *quantum_info)
+%
+%  A description of each parameter follows:
+%
+%    o quantum_info: the quantum info.
+%
+*/
+MagickExport QuantumFormatType GetQuantumFormat(const QuantumInfo *quantum_info)
+{
+  assert(quantum_info != (QuantumInfo *) NULL);
+  assert(quantum_info->signature == MagickCoreSignature);
+  return(quantum_info->format);
 }
 
 /*
@@ -247,25 +427,25 @@ MagickExport void GetQuantumInfo(const ImageInfo *image_info,
     *option;
 
   assert(quantum_info != (QuantumInfo *) NULL);
-  (void) ResetMagickMemory(quantum_info,0,sizeof(*quantum_info));
+  (void) memset(quantum_info,0,sizeof(*quantum_info));
   quantum_info->quantum=8;
   quantum_info->maximum=1.0;
   quantum_info->scale=QuantumRange;
   quantum_info->pack=MagickTrue;
   quantum_info->semaphore=AllocateSemaphoreInfo();
-  quantum_info->signature=MagickSignature;
+  quantum_info->signature=MagickCoreSignature;
   if (image_info == (const ImageInfo *) NULL)
     return;
   option=GetImageOption(image_info,"quantum:format");
   if (option != (char *) NULL)
-    quantum_info->format=(QuantumFormatType) ParseMagickOption(
+    quantum_info->format=(QuantumFormatType) ParseCommandOption(
       MagickQuantumFormatOptions,MagickFalse,option);
   option=GetImageOption(image_info,"quantum:minimum");
   if (option != (char *) NULL)
-    quantum_info->minimum=atof(option);
+    quantum_info->minimum=StringToDouble(option,(char **) NULL);
   option=GetImageOption(image_info,"quantum:maximum");
   if (option != (char *) NULL)
-    quantum_info->maximum=atof(option);
+    quantum_info->maximum=StringToDouble(option,(char **) NULL);
   if ((quantum_info->minimum == 0.0) && (quantum_info->maximum == 0.0))
     quantum_info->scale=0.0;
   else
@@ -279,11 +459,13 @@ MagickExport void GetQuantumInfo(const ImageInfo *image_info,
         quantum_info->minimum);
   option=GetImageOption(image_info,"quantum:scale");
   if (option != (char *) NULL)
-    quantum_info->scale=atof(option);
+    quantum_info->scale=StringToDouble(option,(char **) NULL);
   option=GetImageOption(image_info,"quantum:polarity");
   if (option != (char *) NULL)
     quantum_info->min_is_white=LocaleCompare(option,"min-is-white") == 0 ?
       MagickTrue : MagickFalse;
+  quantum_info->endian=image_info->endian;
+  ResetQuantumState(quantum_info);
 }
 
 /*
@@ -311,9 +493,12 @@ MagickExport void GetQuantumInfo(const ImageInfo *image_info,
 */
 MagickExport unsigned char *GetQuantumPixels(const QuantumInfo *quantum_info)
 {
+  const int
+    id = GetOpenMPThreadId();
+
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
-  return(quantum_info->pixels);
+  assert(quantum_info->signature == MagickCoreSignature);
+  return((unsigned char *) GetVirtualMemoryBlob(quantum_info->pixels[id]));
 }
 
 /*
@@ -344,9 +529,10 @@ MagickExport QuantumType GetQuantumType(Image *image,ExceptionInfo *exception)
     quantum_type;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
+  assert(image->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  (void) exception;
   quantum_type=RGBQuantum;
   if (image->matte != MagickFalse)
     quantum_type=RGBAQuantum;
@@ -356,19 +542,18 @@ MagickExport QuantumType GetQuantumType(Image *image,ExceptionInfo *exception)
       if (image->matte != MagickFalse)
         quantum_type=CMYKAQuantum;
     }
-  if (IsGrayImage(image,exception) != MagickFalse)
+  if (IsGrayColorspace(image->colorspace) != MagickFalse)
     {
       quantum_type=GrayQuantum;
       if (image->matte != MagickFalse)
         quantum_type=GrayAlphaQuantum;
     }
-  else
-    if (image->storage_class == PseudoClass)
-      {
-        quantum_type=IndexQuantum;
-        if (image->matte != MagickFalse)
-          quantum_type=IndexAlphaQuantum;
-      }
+  if (image->storage_class == PseudoClass)
+    {
+      quantum_type=IndexQuantum;
+      if (image->matte != MagickFalse)
+        quantum_type=IndexAlphaQuantum;
+    }
   return(quantum_type);
 }
 
@@ -377,6 +562,51 @@ MagickExport QuantumType GetQuantumType(Image *image,ExceptionInfo *exception)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   R e s e t Q u a n t u m S t a t e                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ResetQuantumState() resets the quantum state.
+%
+%  The format of the ResetQuantumState method is:
+%
+%      void ResetQuantumState(QuantumInfo *quantum_info)
+%
+%  A description of each parameter follows:
+%
+%    o quantum_info: the quantum info.
+%
+*/
+MagickPrivate void ResetQuantumState(QuantumInfo *quantum_info)
+{
+  static const unsigned int mask[32] =
+  {
+    0x00000000U, 0x00000001U, 0x00000003U, 0x00000007U, 0x0000000fU,
+    0x0000001fU, 0x0000003fU, 0x0000007fU, 0x000000ffU, 0x000001ffU,
+    0x000003ffU, 0x000007ffU, 0x00000fffU, 0x00001fffU, 0x00003fffU,
+    0x00007fffU, 0x0000ffffU, 0x0001ffffU, 0x0003ffffU, 0x0007ffffU,
+    0x000fffffU, 0x001fffffU, 0x003fffffU, 0x007fffffU, 0x00ffffffU,
+    0x01ffffffU, 0x03ffffffU, 0x07ffffffU, 0x0fffffffU, 0x1fffffffU,
+    0x3fffffffU, 0x7fffffffU
+  };
+
+  assert(quantum_info != (QuantumInfo *) NULL);
+  assert(quantum_info->signature == MagickCoreSignature);
+  quantum_info->state.inverse_scale=1.0;
+  if (fabs(quantum_info->scale) >= MagickEpsilon)
+    quantum_info->state.inverse_scale/=quantum_info->scale;
+  quantum_info->state.pixel=0U;
+  quantum_info->state.bits=0U;
+  quantum_info->state.mask=mask;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   S e t Q u a n t u m F o r m a t                                           %
 %                                                                             %
 %                                                                             %
@@ -401,7 +631,7 @@ MagickExport void SetQuantumAlphaType(QuantumInfo *quantum_info,
   const QuantumAlphaType type)
 {
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->alpha_type=type;
 }
 
@@ -420,8 +650,8 @@ MagickExport void SetQuantumAlphaType(QuantumInfo *quantum_info,
 %
 %  The format of the SetQuantumDepth method is:
 %
-%      unsigned char *SetQuantumDepth(const Image *image,
-%        QuantumInfo *quantum_info,const unsigned long depth)
+%      MagickBooleanType SetQuantumDepth(const Image *image,
+%        QuantumInfo *quantum_info,const size_t depth)
 %
 %  A description of each parameter follows:
 %
@@ -432,42 +662,91 @@ MagickExport void SetQuantumAlphaType(QuantumInfo *quantum_info,
 %    o depth: the quantum depth.
 %
 */
-MagickExport unsigned char *SetQuantumDepth(const Image *image,
-  QuantumInfo *quantum_info,const unsigned long depth)
+MagickExport MagickBooleanType SetQuantumDepth(const Image *image,
+  QuantumInfo *quantum_info,const size_t depth)
 {
+  size_t
+    extent,
+    quantum;
+
   /*
     Allocate the quantum pixel buffer.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(image->signature == MagickCoreSignature);
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
-  quantum_info->depth=depth;
+  assert(quantum_info->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  quantum_info->depth=MagickMin(depth,64);
   if (quantum_info->format == FloatingPointQuantumFormat)
     {
       if (quantum_info->depth > 32)
         quantum_info->depth=64;
       else
-        quantum_info->depth=32;
+        if (quantum_info->depth > 24)
+          quantum_info->depth=32;
+        else
+          if (quantum_info->depth > 16)
+            quantum_info->depth=24;
+          else
+            quantum_info->depth=16;
     }
-  if (quantum_info->pixels != (unsigned char *) NULL)
+  /*
+    Speculative allocation since we don't yet know the quantum type.
+  */
+  quantum=(quantum_info->pad+6)*((quantum_info->depth+7)/8)*sizeof(double);
+  extent=MagickMax(image->columns,image->rows)*quantum;
+  if ((MagickMax(image->columns,image->rows) != 0) &&
+      (quantum != (extent/MagickMax(image->columns,image->rows))))
+    return(MagickFalse);
+  if (quantum_info->pixels != (MemoryInfo **) NULL)
     {
-      assert(quantum_info->pixels[quantum_info->extent] == QuantumSignature);
-      quantum_info->pixels=(unsigned char *) RelinquishMagickMemory(
-        quantum_info->pixels);
+      if (extent <= quantum_info->extent)
+        return(MagickTrue);
+      DestroyQuantumPixels(quantum_info);
     }
-  quantum_info->pixels=(unsigned char *) AcquireQuantumMemory(
-    (quantum_info->pad+5)*image->columns+1,((quantum_info->depth+7)/8)*
-    sizeof(*quantum_info->pixels));
-  if (quantum_info->pixels == (unsigned char *) NULL)
-    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  quantum_info->extent=(quantum_info->pad+5)*image->columns*
-    ((quantum_info->depth+7)/8);
-  (void) ResetMagickMemory(quantum_info->pixels,0,quantum_info->extent);
-  quantum_info->pixels[quantum_info->extent]=QuantumSignature;
-  return(quantum_info->pixels);
+  return(AcquireQuantumPixels(quantum_info,extent));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   S e t Q u a n t u m E n d i a n                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetQuantumEndian() sets the quantum endian.
+%
+%  The endian of the SetQuantumEndian method is:
+%
+%      MagickBooleanType SetQuantumEndian(const Image *image,
+%        QuantumInfo *quantum_info,const EndianType endian)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o quantum_info: the quantum info.
+%
+%    o endian: the quantum endian.
+%
+*/
+MagickExport MagickBooleanType SetQuantumEndian(const Image *image,
+  QuantumInfo *quantum_info,const EndianType endian)
+{
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickCoreSignature);
+  assert(quantum_info != (QuantumInfo *) NULL);
+  assert(quantum_info->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  quantum_info->endian=endian;
+  return(SetQuantumDepth(image,quantum_info,quantum_info->depth));
 }
 
 /*
@@ -485,7 +764,7 @@ MagickExport unsigned char *SetQuantumDepth(const Image *image,
 %
 %  The format of the SetQuantumFormat method is:
 %
-%      unsigned char *SetQuantumFormat(const Image *image,
+%      MagickBooleanType SetQuantumFormat(const Image *image,
 %        QuantumInfo *quantum_info,const QuantumFormatType format)
 %
 %  A description of each parameter follows:
@@ -497,15 +776,15 @@ MagickExport unsigned char *SetQuantumDepth(const Image *image,
 %    o format: the quantum format.
 %
 */
-MagickExport unsigned char *SetQuantumFormat(const Image *image,
+MagickExport MagickBooleanType SetQuantumFormat(const Image *image,
   QuantumInfo *quantum_info,const QuantumFormatType format)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(image->signature == MagickCoreSignature);
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   quantum_info->format=format;
   return(SetQuantumDepth(image,quantum_info,quantum_info->depth));
 }
@@ -540,8 +819,8 @@ MagickExport void SetQuantumImageType(Image *image,
   const QuantumType quantum_type)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
+  assert(image->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   switch (quantum_type)
   {
@@ -606,11 +885,10 @@ MagickExport void SetQuantumPack(QuantumInfo *quantum_info,
   const MagickBooleanType pack)
 {
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->pack=pack;
 }
 
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -626,8 +904,8 @@ MagickExport void SetQuantumPack(QuantumInfo *quantum_info,
 %
 %  The format of the SetQuantumPad method is:
 %
-%      unsigned char *SetQuantumPad(const Image *image,
-%        QuantumInfo *quantum_info,const unsigned long pad)
+%      MagickBooleanType SetQuantumPad(const Image *image,
+%        QuantumInfo *quantum_info,const size_t pad)
 %
 %  A description of each parameter follows:
 %
@@ -638,15 +916,17 @@ MagickExport void SetQuantumPack(QuantumInfo *quantum_info,
 %    o pad: the quantum pad.
 %
 */
-MagickExport unsigned char *SetQuantumPad(const Image *image,
-  QuantumInfo *quantum_info,const unsigned long pad)
+MagickExport MagickBooleanType SetQuantumPad(const Image *image,
+  QuantumInfo *quantum_info,const size_t pad)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(image->signature == MagickCoreSignature);
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  if (pad >= (MAGICK_SSIZE_MAX/5))
+    return(MagickFalse);
   quantum_info->pad=pad;
   return(SetQuantumDepth(image,quantum_info,quantum_info->depth));
 }
@@ -680,7 +960,7 @@ MagickExport void SetQuantumMinIsWhite(QuantumInfo *quantum_info,
   const MagickBooleanType min_is_white)
 {
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->min_is_white=min_is_white;
 }
 
@@ -699,8 +979,7 @@ MagickExport void SetQuantumMinIsWhite(QuantumInfo *quantum_info,
 %
 %  The format of the SetQuantumQuantum method is:
 %
-%      void SetQuantumQuantum(QuantumInfo *quantum_info,
-%        const unsigned long quantum)
+%      void SetQuantumQuantum(QuantumInfo *quantum_info,const size_t quantum)
 %
 %  A description of each parameter follows:
 %
@@ -710,10 +989,10 @@ MagickExport void SetQuantumMinIsWhite(QuantumInfo *quantum_info,
 %
 */
 MagickExport void SetQuantumQuantum(QuantumInfo *quantum_info,
-  const unsigned long quantum)
+  const size_t quantum)
 {
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->quantum=quantum;
 }
 
@@ -744,6 +1023,6 @@ MagickExport void SetQuantumQuantum(QuantumInfo *quantum_info,
 MagickExport void SetQuantumScale(QuantumInfo *quantum_info,const double scale)
 {
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->scale=scale;
 }
