@@ -10,20 +10,20 @@
 %                              V    M   M  SSSSS                              %
 %                                                                             %
 %                                                                             %
-%                     VMS Utility Methods for ImageMagick                     %
+%                         MagickCore VMS Utility Methods                      %
 %                                                                             %
 %                               Software Design                               %
-%                                 John Cristy                                 %
+%                                    Cristy                                   %
 %                                October 1994                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -44,6 +44,7 @@
 */
 #include "magick/studio.h"
 #include "magick/string_.h"
+#include "magick/memory_.h"
 #include "magick/vms.h"
 
 #if !defined(_AXP_) && (!defined(__VMS_VER) || (__VMS_VER < 70000000))
@@ -71,11 +72,11 @@
 */
 void closedir(DIR *directory)
 {
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(directory != (DIR *) NULL);
-  directory->pattern=(char *) RelinquishMagickMemory(directory->pattern);
-  directory=(char *) RelinquishMagickMemory(directory);
+  directory->pattern=DestroyString(directory->pattern);
+  directory=DestroyString(directory);
 }
 
 /*
@@ -118,17 +119,18 @@ DIR *opendir(char *name)
     }
   if (strcmp(".",name) == 0)
     name="";
-  directory->pattern=(char *) AcquireMagickMemory(strlen(name)+sizeof("*.*")+1);
+  directory->pattern=(char *) AcquireQuantumMemory(strlen(name)+sizeof("*.*")+
+    1UL,sizeof(*directory->pattern));
   if (directory->pattern == (char *) NULL)
     {
-      directory=(char *) RelinquishMagickMemory(directory);
+      directory=DestroyString(directory);
       errno=ENOMEM;
       return(NULL);
     }
   /*
     Initialize descriptor.
   */
-  (void) FormatMagickString(directory->pattern,MaxTextExtent,"%s*.*",name);
+  (void) FormatLocaleString(directory->pattern,MaxTextExtent,"%s*.*",name);
   directory->context=0;
   directory->pat.dsc$a_pointer=directory->pattern;
   directory->pat.dsc$w_length=strlen(directory->pattern);
@@ -169,10 +171,10 @@ struct dirent *readdir(DIR *directory)
   int
     status;
 
-  register char
+  char
     *p;
 
-  register int
+  int
     i;
 
   struct dsc$descriptor_s
@@ -193,13 +195,13 @@ struct dirent *readdir(DIR *directory)
   */
   buffer[sizeof(buffer)-1]='\0';
   for (p=buffer; *p; p++)
-    if (isupper((unsigned char) *p))
-      *p=tolower(*p);
+    if (isupper((int) ((unsigned char) *p)))
+      *p=LocaleToLowercase(*p);
   /*
     Skip any directory component and just copy the name.
   */
   p=buffer;
-  while (isspace((unsigned char) *p) == 0)
+  while (isspace((int) ((unsigned char) *p)) == 0)
     p++;
   *p='\0';
   p=strchr(buffer,']');
@@ -244,7 +246,7 @@ MagickExport MagickBooleanType VMSIsMagickConflict(const char *magick)
     item_list[2];
 
   int
-    device_class,
+    device_class = 0,
     status;
 
   struct dsc$descriptor_s
@@ -259,7 +261,7 @@ MagickExport MagickBooleanType VMSIsMagickConflict(const char *magick)
   item_list[0].ile3$w_code=DVI$_DEVCLASS;
   item_list[0].ile3$ps_bufaddr=&device_class;
   item_list[0].ile3$ps_retlen_addr=NULL;
-  (void) ResetMagickMemory(&item_list[1],0,sizeof(item_list[1]));
+  (void) memset(&item_list[1],0,sizeof(item_list[1]));
   status=sys$getdviw(0,0,&device,&item_list,0,0,0,0);
   if ((status == SS$_NONLOCAL) ||
       ((status & 0x01) && (device_class & (DC$_DISK | DC$_TAPE))))
